@@ -452,34 +452,6 @@ char *ioc_name(unsigned int major, unsigned int minor)
 	return (name);
 }
 
-/*
- ***************************************************************************
- * Check whether a device is a whole disk device or not.
- *
- * IN:
- * @major	Device major number.
- * @minor	Device minor number.
- *
- * RETURNS:
- * Predicate: Returns 1 if dev (major,minor) is a whole disk device.
- *            Returns 0 otherwise.
- ***************************************************************************
- */
-int ioc_iswhole(unsigned int major, unsigned int minor)
-{
-	if (!ioc_parsed && !ioc_init())
-		return 0;
-
-	if (major > MAX_BLKDEV)
-		return 0;
-
-	if (ioconf[major] == NULL)
-		/* Device not registered */
-		return 0;
-
-	return (IS_WHOLE(major, minor));
-}
-
 /*
  ***************************************************************************
  * Transform device mapper name: Get the user assigned name of the logical
@@ -517,8 +489,8 @@ char *transform_devmapname(unsigned int major, unsigned int minor)
 		if (__stat(filen, &aux) == 0) {
 			/* Get its minor and major numbers */
 
-			dm_major = major(aux.st_rdev);
-			dm_minor = minor(aux.st_rdev);
+			dm_major = __major(aux.st_rdev);
+			dm_minor = __minor(aux.st_rdev);
 
 			if ((dm_minor == minor) && (dm_major == major)) {
 				strncpy(name, dp->d_name, sizeof(name));
