@@ -1,6 +1,6 @@
 /*
- * json_stats.c: Funtions used by sadf to display statistics in JSON format.
- * (C) 1999-2020 by Sebastien GODARD (sysstat <at> orange.fr)
+ * json_stats.c: Functions used by sadf to display statistics in JSON format.
+ * (C) 1999-2021 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -35,7 +35,7 @@
 #define _(string) (string)
 #endif
 
-extern unsigned int flags;
+extern uint64_t flags;
 
 /*
  ***************************************************************************
