@@ -1,6 +1,6 @@
 /*
  * rd_stats.h: Include file used to read system statistics
- * (C) 1999-2020 by Sebastien Godard (sysstat <at> orange.fr)
+ * (C) 1999-2021 by Sebastien Godard (sysstat <at> orange.fr)
  */
 
 #ifndef _RD_STATS_H
@@ -14,16 +14,10 @@
  ***************************************************************************
  */
 
-/* Get IFNAMSIZ */
-#include <net/if.h>
-#ifndef IFNAMSIZ
-#define IFNAMSIZ	16
-#endif
-
 /* Maximum length of block device name */
 #define MAX_DEV_LEN	128
 /* Maximum length of network interface name */
-#define MAX_IFACE_LEN	IFNAMSIZ
+#define MAX_IFACE_LEN	16
 /* Maximum length of USB manufacturer string */
 #define MAX_MANUF_LEN	24
 /* Maximum length of USB product string */
