@@ -1,6 +1,6 @@
 /*
- * rndr_stats.c: Funtions used by sadf to display statistics in selected format.
- * (C) 1999-2020 by Sebastien GODARD (sysstat <at> orange.fr)
+ * rndr_stats.c: Functions used by sadf to display statistics in selected format.
+ * (C) 1999-2021 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -37,7 +37,7 @@
 
 char *seps[] =  {"\t", ";"};
 
-extern unsigned int flags;
+extern uint64_t flags;
 
 /*
  ***************************************************************************
