@@ -1,6 +1,6 @@
 /*
  * sar and sadf common routines.
- * (C) 1999-2020 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2021 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -198,13 +198,9 @@ void guess_sa_name(char *sa_dir, struct tm *rectime, int *sa_name)
  *
  * OUT:
  * @datafile	Name of daily data file.
- *
- * RETURNS:
- * 1 if an output error has been encountered or if datafile name has been
- * truncated, or 0 otherwise.
  ***************************************************************************
  */
-int set_default_file(char *datafile, int d_off, int sa_name)
+void set_default_file(char *datafile, int d_off, int sa_name)
 {
 	char sa_dir[MAX_FILE_LEN];
 	struct tm rectime = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL};
@@ -243,13 +239,17 @@ int set_default_file(char *datafile, int d_off, int sa_name)
 			       rectime.tm_mday);
 	}
 	datafile[MAX_FILE_LEN - 1] = '\0';
+
+	if ((err < 0) || (err >= MAX_FILE_LEN)) {
+		fprintf(stderr, "%s: %s\n", __FUNCTION__, datafile);
+		exit(1);
+	}
+
 	default_file_used = TRUE;
 
 #ifdef DEBUG
 	fprintf(stderr, "%s: Datafile: %s\n", __FUNCTION__, datafile);
 #endif
-
-	return ((err < 0) || (err >= MAX_FILE_LEN));
 }
 
 /*
@@ -330,6 +330,8 @@ void display_sa_file_version(FILE *st, struct file_magic *file_magic)
 void handle_invalid_sa_file(int fd, struct file_magic *file_magic, char *file,
 			    int n)
 {
+	unsigned short fmt_magic;
+
 	fprintf(stderr, _("Invalid system activity file: %s\n"), file);
 
 	if (n == FILE_MAGIC_SIZE) {
@@ -337,10 +339,19 @@ void handle_invalid_sa_file(int fd, struct file_magic *file_magic, char *file,
 			/* This is a sysstat file, but this file has an old format */
 			display_sa_file_version(stderr, file_magic);
 
+			fmt_magic = file_magic->sysstat_magic == SYSSTAT_MAGIC ?
+				    file_magic->format_magic : __builtin_bswap16(file_magic->format_magic);
 			fprintf(stderr,
 				_("Current sysstat version cannot read the format of this file (%#x)\n"),
-				file_magic->sysstat_magic == SYSSTAT_MAGIC ?
-				file_magic->format_magic : __builtin_bswap16(file_magic->format_magic));
+				fmt_magic);
+			if (fmt_magic >= FORMAT_MAGIC_2171) {
+				fprintf(stderr,
+					_("Try to convert it to current format. Enter:\n\n"));
+				fprintf(stderr, "sadf -c %s > %s.new\n\n", file, file);
+				fprintf(stderr,
+					_("You should then be able to read the new file created (%s.new)\n"),
+					file);
+			}
 		}
 	}
 
@@ -570,15 +581,11 @@ int next_slice(unsigned long long uptime_ref, unsigned long long uptime,
 	/*
 	 * A few notes about the "algorithm" used here to display selected entries
 	 * from the system activity file (option -f with -i flag):
-	 * Let 'Iu' be the interval value given by the user on the command line,
-	 *     'If' the interval between current and previous line in the system
-	 * activity file,
-	 * and 'En' the nth entry (identified by its time stamp) of the file.
-	 * We choose In = [ En - If/2, En + If/2 [ if If is even,
-	 *        or In = [ En - If/2, En + If/2 ] if not.
-	 * En will be displayed if
-	 *       (Pn * Iu) or (P'n * Iu) belongs to In
-	 * with  Pn = En / Iu and P'n = En / Iu + 1
+	 * Let Iu be the interval value given by the user on the command line,
+	 *     In the interval between current and previous sample,
+	 * and En the current sample (identified by its time stamp) in the file.
+	 * En will ne displayed if there is an integer p so that:
+	 * p * Iu belongs to [En - In/2, En + In/2[.
 	 */
 	f = ((double) ((uptime - uptime_ref) & 0xffffffff)) / 100;
 	entry = (unsigned long) f;
@@ -1456,6 +1463,7 @@ int skip_extra_struct(int ifd, int endian_mismatch, int arch_64)
  *		sadf stop. Default behavior is to stop on unexpected EOF.
  * @b_size	@buffer size.
  * @flags	Flags for common options and system state.
+ * @ofmt	Pointer on report output format structure.
  *
  * OUT:
  * @record_hdr	Record header for current sample.
@@ -1468,7 +1476,7 @@ int skip_extra_struct(int ifd, int endian_mismatch, int arch_64)
  */
 int read_record_hdr(int ifd, void *buffer, struct record_header *record_hdr,
 		    struct file_header *file_hdr, int arch_64, int endian_mismatch,
-		    int oneof, size_t b_size, uint64_t flags)
+		    int oneof, size_t b_size, uint64_t flags, struct report_format *ofmt)
 {
 	int rc;
 
@@ -1489,13 +1497,24 @@ int read_record_hdr(int ifd, void *buffer, struct record_header *record_hdr,
 		}
 
 		/* Raw output in debug mode */
-		if (DISPLAY_DEBUG_MODE(flags)) {
+		if (DISPLAY_DEBUG_MODE(flags) && (ofmt->id == F_RAW_OUTPUT)) {
 			printf("# uptime_cs; %llu; ust_time; %llu; extra_next; %u; record_type; %d; HH:MM:SS; %02d:%02d:%02d\n",
 			       record_hdr->uptime_cs, record_hdr->ust_time,
 			       record_hdr->extra_next, record_hdr->record_type,
 			       record_hdr->hour, record_hdr->minute, record_hdr->second);
 		}
 
+		/* Sanity checks */
+		if ((record_hdr->record_type <= 0) || (record_hdr->record_type > R_EXTRA_MAX) ||
+		    (record_hdr->hour > 23) || (record_hdr->minute > 59) || (record_hdr->second > 60)) {
+#ifdef DEBUG
+			fprintf(stderr, "%s: record_type=%d HH:MM:SS=%02d:%02d:%02d\n",
+				__FUNCTION__, record_hdr->record_type,
+				record_hdr->hour, record_hdr->minute, record_hdr->second);
+#endif
+			return 2;
+		}
+
 		/*
 		 * Skip unknown extra structures if present.
 		 * This will be done later for R_COMMENT and R_RESTART records, as extra structures
@@ -1977,7 +1996,8 @@ void check_file_actlst(int *ifd, char *dfile, struct activity *act[], uint64_t f
 
 		/*
 		 * Every activity, known or unknown, should have
-		 * at least one item and sub-item, and a positive size value.
+		 * at least one item and sub-item, and a size value in
+		 * a defined range.
 		 * Also check that the number of items and sub-items
 		 * doesn't exceed a max value. This is necessary
 		 * because we will use @nr and @nr2 to
@@ -1988,10 +2008,10 @@ void check_file_actlst(int *ifd, char *dfile, struct activity *act[], uint64_t f
 		 */
 		if ((fal->nr < 1) || (fal->nr2 < 1) ||
 		    (fal->nr > NR_MAX) || (fal->nr2 > NR2_MAX) ||
-		    (fal->size <= 0)) {
+		    (fal->size <= 0) || (fal->size > MAX_ITEM_STRUCT_SIZE)) {
 #ifdef DEBUG
-			fprintf(stderr, "%s: id=%d nr=%d nr2=%d\n",
-				__FUNCTION__, fal->id, fal->nr, fal->nr2);
+			fprintf(stderr, "%s: id=%d nr=%d nr2=%d size=%d\n",
+				__FUNCTION__, fal->id, fal->nr, fal->nr2, fal->size);
 #endif
 			goto format_error;
 		}
@@ -2657,39 +2677,6 @@ void set_bitmaps(struct activity *act[], uint64_t *flags)
 	}
 }
 
-/*
- ***************************************************************************
- * Count number of comma-separated values in arguments list. For example,
- * the number will be 3 for "sar --dev=sda,sdb,sdc -dp 2 5", 1 for
- * "sar --dev=sda -dp 2 5" and 0 for "sar --dev= -dp 2 5".
- *
- * IN:
- * @arg_v	Argument containing the list of coma-separated values.
- *
- * RETURNS:
- * Number of comma-separated values in the list.
- ***************************************************************************
- */
-int count_csval_arg(char *arg_v)
-{
-	int nr = 0;
-	char *t;
-
-	if (arg_v[0] == '\0')
-		return 0;
-
-	if (strchr(arg_v, ',')) {
-		for (t = arg_v; t; t = strchr(t + 1, ',')) {
-			nr++;
-		}
-	}
-	if (!nr) {
-		nr = 1;
-	}
-
-	return nr;
-}
-
 /*
  ***************************************************************************
  * Look for item in list.
@@ -2870,10 +2857,8 @@ void replace_nonprintable_char(int ifd, char *comment)
  *
  * OUT:
  * @rectime	Structure where timestamp for current record has been saved
- * 		(in local time or in UTC depending on options used).
- * @loctime	If given, structure where timestamp for current record has
- * 		been saved (expressed in local time). This field will be used
- * 		for time comparison if options -s and/or -e have been used.
+ * 		(in local time, in UTC or in time of file's creator
+ * 		depending on options used).
  *
  * RETURNS:
  * 1 if an error was detected, or 0 otherwise.
