@@ -1,6 +1,6 @@
 /*
  * sadf: system activity data formatter
- * (C) 1999-2020 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2021 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -230,7 +230,7 @@ void check_format_options(void)
  *
  * RETURNS:
  * 1 if EOF has been reached,
- * 2 if an unexpected EOF has been reached,
+ * 2 if an unexpected EOF has been reached, or an error occurred.
  * 0 otherwise.
  ***************************************************************************
  */
@@ -243,7 +243,8 @@ int read_next_sample(int ifd, int action, int curr, char *file, int *rtype, int
 
 	/* Read current record */
 	if ((rc = read_record_hdr(ifd, rec_hdr_tmp, &record_hdr[curr], &file_hdr,
-			    arch_64, endian_mismatch, oneof, sizeof(rec_hdr_tmp), flags)) != 0)
+				  arch_64, endian_mismatch, oneof, sizeof(rec_hdr_tmp), flags,
+				  fmt[f_position])) != 0)
 		/* End of sa file */
 		return rc;
 
@@ -264,8 +265,9 @@ int read_next_sample(int ifd, int action, int curr, char *file, int *rtype, int
 				return 2;
 
 			if (action & SET_TIMESTAMPS) {
-				sa_get_record_timestamp_struct(flags, &record_hdr[curr],
-							       rectime);
+				if (sa_get_record_timestamp_struct(flags, &record_hdr[curr],
+								   rectime))
+					return 2;
 			}
 		}
 		else {
@@ -292,7 +294,8 @@ int read_next_sample(int ifd, int action, int curr, char *file, int *rtype, int
 					return 2;
 			}
 			if (action & SET_TIMESTAMPS) {
-				sa_get_record_timestamp_struct(flags, &record_hdr[curr], rectime);
+				if (sa_get_record_timestamp_struct(flags, &record_hdr[curr], rectime))
+					return 2;
 			}
 		}
 		else {
@@ -311,7 +314,8 @@ int read_next_sample(int ifd, int action, int curr, char *file, int *rtype, int
 		if (read_file_stat_bunch(act, curr, ifd, file_hdr.sa_act_nr, file_actlst,
 					 endian_mismatch, arch_64, file, file_magic, oneof) > 0)
 			return 2;
-		sa_get_record_timestamp_struct(flags, &record_hdr[curr], rectime);
+		if (sa_get_record_timestamp_struct(flags, &record_hdr[curr], rectime))
+			return 2;
 	}
 
 	return 0;
@@ -655,7 +659,7 @@ int generic_write_stats(int curr, int use_tm_start, int use_tm_end, int reset,
 	if (reset_cd) {
 		/*
 		 * See note in sar.c.
-		 * NB: Reseting cross_day is needed only if datafile
+		 * NB: Resetting cross_day is needed only if datafile
 		 * may be rewinded (eg. in db or ppc output formats).
 		 */
 		cross_day = 0;
@@ -681,20 +685,15 @@ int generic_write_stats(int curr, int use_tm_start, int use_tm_end, int reset,
 	}
 
 	/* Check time (2) */
-	if (use_tm_start && (datecmp(rectime, &tm_start, cross_day) < 0))
-		/* it's too soon... */
-		return 0;
-
-	/* Get interval values in 1/100th of a second */
-	get_itv_value(&record_hdr[curr], &record_hdr[!curr], &itv);
-
-	/* Check time (3) */
 	if (use_tm_end && (datecmp(rectime, &tm_end, cross_day) > 0)) {
-		/* It's too late... */
+		/* End time exceeded */
 		*cnt = 0;
 		return 0;
 	}
 
+	/* Get interval values in 1/100th of a second */
+	get_itv_value(&record_hdr[curr], &record_hdr[!curr], &itv);
+
 	dt = itv / 100;
 	/* Correct rounding error for dt */
 	if ((itv % 100) >= 50) {
@@ -717,13 +716,14 @@ int generic_write_stats(int curr, int use_tm_start, int use_tm_end, int reset,
 			continue;
 
 		if ((TEST_MARKUP(fmt[f_position]->options) && CLOSE_MARKUP(act[i]->options)) ||
-		    (IS_SELECTED(act[i]->options) && (act[i]->nr > 0))) {
+		    (IS_SELECTED(act[i]->options) && (act[i]->nr[curr] > 0)) ||
+		    (format == F_RAW_OUTPUT)) {
 
 			if (format == F_JSON_OUTPUT) {
 				/* JSON output */
 				int *tab = (int *) parm;
 
-				if (IS_SELECTED(act[i]->options) && (act[i]->nr > 0)) {
+				if (IS_SELECTED(act[i]->options) && (act[i]->nr[curr] > 0)) {
 
 					if (*fmt[f_position]->f_timestamp) {
 						(*fmt[f_position]->f_timestamp)(tab, F_MAIN, cur_date, cur_time,
@@ -756,13 +756,15 @@ int generic_write_stats(int curr, int use_tm_start, int use_tm_end, int reset,
 					       act[i]->nr[curr], act[i]->nr_allocated, act[i]->nr_ini);
 				}
 
-				(*act[i]->f_raw_print)(act[i], pre, curr);
+				if (IS_SELECTED(act[i]->options) && (act[i]->nr[curr] > 0)) {
+					(*act[i]->f_raw_print)(act[i], pre, curr);
+				}
 			}
 
 			else if (format == F_PCP_OUTPUT) {
 				/* PCP archive */
 				if (*act[i]->f_pcp_print) {
-					(*act[i]->f_pcp_print)(act[i], curr, itv, &record_hdr[curr]);
+					(*act[i]->f_pcp_print)(act[i], curr);
 				}
 			}
 
@@ -837,7 +839,10 @@ void rw_curr_act_stats(int ifd, int *curr, long *cnt, int *eosaf,
 					  *curr, file, &rtype, 0, file_magic,
 					  file_actlst, rectime, UEOF_STOP);
 
-		if (!*eosaf && (rtype != R_RESTART) && (rtype != R_COMMENT)) {
+		if (*eosaf || (rtype == R_RESTART))
+			break;
+
+		if (rtype != R_COMMENT) {
 			next = generic_write_stats(*curr, tm_start.use, tm_end.use, *reset, cnt,
 						   NULL, rectime, reset_cd, act_id);
 			reset_cd = 0;
@@ -856,7 +861,7 @@ void rw_curr_act_stats(int ifd, int *curr, long *cnt, int *eosaf,
 			*reset = FALSE;
 		}
 	}
-	while (*cnt && !*eosaf && (rtype != R_RESTART));
+	while (*cnt);
 
 	*reset = TRUE;
 }
@@ -902,7 +907,8 @@ void display_curr_act_graphs(int ifd, int *curr, long *cnt, int *eosaf,
 
 	/*
 	 * Restore the first stats collected.
-	 * Used to compute the rate displayed on the first line.
+	 * Originally used to compute the rate displayed on the first line.
+	 * Here, this is to plot the first point and start the graph.
 	 */
 	copy_structures(act, id_seq, record_hdr, !*curr, 2);
 
@@ -923,8 +929,23 @@ void display_curr_act_graphs(int ifd, int *curr, long *cnt, int *eosaf,
 		*eosaf = read_next_sample(ifd, IGNORE_RESTART | IGNORE_COMMENT | SET_TIMESTAMPS,
 					  *curr, file, &rtype, 0, file_magic,
 					  file_actlst, rectime, UEOF_CONT);
+		if (*eosaf)
+			break;
 
-		if (!*eosaf && (rtype != R_COMMENT) && (rtype != R_RESTART)) {
+		if (rtype == R_RESTART) {
+			parm.restart = TRUE;
+			*reset = TRUE;
+			/* Go to next statistics record, if possible */
+			do {
+				*eosaf = read_next_sample(ifd, IGNORE_RESTART | IGNORE_COMMENT | SET_TIMESTAMPS,
+							  *curr, file, &rtype, 0, file_magic,
+							  file_actlst, rectime, UEOF_CONT);
+			}
+			while (!*eosaf && ((rtype == R_RESTART) || (rtype == R_COMMENT)));
+
+			*curr ^= 1;
+		}
+		else if (rtype != R_COMMENT) {
 
 			next = generic_write_stats(*curr, tm_start.use, tm_end.use, *reset, cnt,
 						   &parm, rectime, reset_cd, act[p]->id);
@@ -944,21 +965,8 @@ void display_curr_act_graphs(int ifd, int *curr, long *cnt, int *eosaf,
 			}
 			*reset = FALSE;
 		}
-		if (!*eosaf && (rtype == R_RESTART)) {
-			parm.restart = TRUE;
-			*reset = TRUE;
-			/* Go to next statistics record, if possible */
-			do {
-				*eosaf = read_next_sample(ifd, IGNORE_RESTART | IGNORE_COMMENT | SET_TIMESTAMPS,
-							  *curr, file, &rtype, 0, file_magic,
-							  file_actlst, rectime, UEOF_CONT);
-			}
-			while (!*eosaf && ((rtype == R_RESTART) || (rtype == R_COMMENT)));
-
-			*curr ^= 1;
-		}
 	}
-	while (!*eosaf);
+	while (!*eosaf && *cnt);
 
 	*reset = TRUE;
 
@@ -1061,47 +1069,47 @@ void logic1_display_loop(int ifd, char *file, struct file_activity *file_actlst,
 		cnt = count;
 		reset = TRUE;
 
-		if (!eosaf) {
+		if (eosaf)
+			break;
 
-			/* Save the first stats collected. Used for example in next_slice() function */
-			copy_structures(act, id_seq, record_hdr, 2, 0);
+		/* Save the first stats collected. Used for example in next_slice() function */
+		copy_structures(act, id_seq, record_hdr, 2, 0);
 
+		do {
+			eosaf = read_next_sample(ifd, ign_flag, curr, file,
+						 &rtype, tab, file_magic, file_actlst,
+						 rectime, UEOF_CONT);
+
+			if (!eosaf && (rtype != R_COMMENT) && (rtype != R_RESTART)) {
+				if (*fmt[f_position]->f_statistics) {
+					(*fmt[f_position]->f_statistics)(&tab, F_MAIN, act, id_seq);
+				}
+
+				/* next is set to 1 when we were close enough to desired interval */
+				next = generic_write_stats(curr, tm_start.use, tm_end.use, reset,
+							  &cnt, &tab, rectime, FALSE, ALL_ACTIVITIES);
+
+				if (next) {
+					curr ^= 1;
+					if (cnt > 0) {
+						cnt--;
+					}
+				}
+				reset = FALSE;
+			}
+		}
+		while (cnt && !eosaf && (rtype != R_RESTART));
+
+		if (!cnt) {
+			/* Go to next Linux restart, if possible */
 			do {
 				eosaf = read_next_sample(ifd, ign_flag, curr, file,
 							 &rtype, tab, file_magic, file_actlst,
 							 rectime, UEOF_CONT);
-
-				if (!eosaf && (rtype != R_COMMENT) && (rtype != R_RESTART)) {
-					if (*fmt[f_position]->f_statistics) {
-						(*fmt[f_position]->f_statistics)(&tab, F_MAIN, act, id_seq);
-					}
-
-					/* next is set to 1 when we were close enough to desired interval */
-					next = generic_write_stats(curr, tm_start.use, tm_end.use, reset,
-								  &cnt, &tab, rectime, FALSE, ALL_ACTIVITIES);
-
-					if (next) {
-						curr ^= 1;
-						if (cnt > 0) {
-							cnt--;
-						}
-					}
-					reset = FALSE;
-				}
 			}
-			while (cnt && !eosaf && (rtype != R_RESTART));
-
-			if (!cnt) {
-				/* Go to next Linux restart, if possible */
-				do {
-					eosaf = read_next_sample(ifd, ign_flag, curr, file,
-								 &rtype, tab, file_magic, file_actlst,
-								 rectime, UEOF_CONT);
-				}
-				while (!eosaf && (rtype != R_RESTART));
-			}
-			reset = TRUE;
+			while (!eosaf && (rtype != R_RESTART));
 		}
+		reset = TRUE;
 	}
 	while (!eosaf);
 
@@ -1599,7 +1607,7 @@ int main(int argc, char **argv)
 					flags |= S_F_SVG_SHOW_INFO;
 				}
 				else if (!strcmp(t, K_DEBUG)) {
-					flags |= S_F_RAW_DEBUG_MODE;
+					flags |= S_F_DEBUG_MODE;
 				}
 				else if (!strncmp(t, K_HEIGHT, strlen(K_HEIGHT))) {
 					v = t + strlen(K_HEIGHT);
